#pragma once

#include "../base/MemoryBase.h"

#include "CoreSegment.h"

#include "../workingmemory/Memory.h"

namespace armarx::armem::ltm
{
    /**
     * @brief Data of a memory consisting of multiple core segments.
     */
    class Memory :
        public base::MemoryBase<CoreSegment, Memory>
    {
        using Base = base::MemoryBase<CoreSegment, Memory>;

    public:

        using Base::MemoryBase;
        using Base::operator=;

        enum class PeriodicTransferMode
        {
            MANUAL,
            TRANSFER_PERIODIC_KEEP,
            TRANSFER_PERIODIC_DELETE,
        };

        enum class FilledTransferMode
        {
            TRANSFER_LATEST,
            TRANSFER_LEAST_USED
        };

        struct PeriodicTransfer
        {
            PeriodicTransferMode mode = PeriodicTransferMode::MANUAL;
        };

        struct FilledTransfer
        {
            FilledTransferMode mode = FilledTransferMode::TRANSFER_LATEST;
        };


        // Conversion
        wm::Memory convert() const;

        // MongoDB connection
        void reload();
        void append(const wm::Memory&);

    protected:
        virtual void _copySelf(Memory& other) const override
        {
            Base::_copySelf(other);
            other.dbsettings = dbsettings;
        }

        virtual void _copySelfEmpty(Memory& other) const override
        {
            Base::_copySelfEmpty(other);
            other.dbsettings = dbsettings;
        }

    private:
        bool checkConnection() const;

    public:
        MongoDBConnectionManager::MongoDBSettings dbsettings;

        PeriodicTransfer periodic_transfer;
        FilledTransferMode filled_transfer;
    };
}
