/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <typeindex>
#include <typeinfo>
#include <map>
#include <string>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/Config.h>

namespace armarx::aron::type
{
    const std::vector<type::Maybe> ALL_MAYBE_TYPES =
    {
        type::Maybe::eNone,
        type::Maybe::eOptional,
        type::Maybe::eRawPointer,
        type::Maybe::eSharedPointer,
        type::Maybe::eUniquePointer
    };

    enum class Descriptor
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    e##upperType,

        HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
        eUnknown = -1
    };

    const std::vector<type::Descriptor> ALL_ARON_TYPES =
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    type::Descriptor::e##upperType,

        HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
        Descriptor::eUnknown
    };

    const std::map<type::Descriptor, std::string> _descriptor2string =
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { Descriptor::e##upperType, "armarx::aron::type::Descriptor::e" + std::string(#upperType) },

        HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
        {Descriptor::eUnknown, "armarx::aron::type::Descriptor::eUnknown"}
    };

    inline std::string DESCRIPTOR_TO_STRING(const type::Descriptor d)
    {
        return type::_descriptor2string.at(d);
    }

    const std::map<type::Maybe, std::string> _maybe2string =
    {
        {type::Maybe::eNone, "armarx::aron::type::Maybe::eNone"},
        {type::Maybe::eOptional, "armarx::aron::type::Maybe::eOptional"},
        {type::Maybe::eRawPointer, "armarx::aron::type::Maybe::eRawPointer"},
        {type::Maybe::eSharedPointer, "armarx::aron::type::Maybe::eSharedPointer"},
        {type::Maybe::eUniquePointer, "armarx::aron::type::Maybe::eUniquePointer"}
    };

    inline std::string MAYBE_TO_STRING(const type::Maybe d)
    {
        return type::_maybe2string.at(d);
    }

    const std::map<std::string, type::Descriptor> _string2descriptor =
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { "armarx::aron::type::Descriptor::e" + std::string(#upperType), Descriptor::e##upperType }, \
    { std::string(#upperType), Descriptor::e##upperType }, \
    { std::string(#lowerType), Descriptor::e##upperType }, \
    { std::string(#capsType), Descriptor::e##upperType },

        HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
        {"armarx::aron::type::Descriptor::eUnknown", Descriptor::eUnknown},
        {"Unknown", Descriptor::eUnknown},
        {"unknown", Descriptor::eUnknown},
        {"UNKNOWN", Descriptor::eUnknown}
    };

    inline type::Descriptor STRING_TO_DESCRIPTOR(const std::string& s)
    {
        return type::_string2descriptor.at(s);
    }

    inline type::Descriptor DESCRIPTOR_FROM_STRING(const std::string& s)
    {
        return STRING_TO_DESCRIPTOR(s);
    }
}

namespace armarx::aron::data
{
    enum class Descriptor
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    e##upperType,

        HANDLE_ALL_ARON_DATA
#undef RUN_ARON_MACRO
        eUnknown = -1
    };

    const std::vector<data::Descriptor> ALL_ARON_DATA =
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    data::Descriptor::e##upperType,

        HANDLE_ALL_ARON_DATA
#undef RUN_ARON_MACRO
        Descriptor::eUnknown
    };

    const std::map<data::Descriptor, std::string> _descriptor2string =
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { Descriptor::e##upperType, "armarx::aron::data::Descriptor::e" + std::string(#upperType) },

        HANDLE_ALL_ARON_DATA
#undef RUN_ARON_MACRO
        {Descriptor::eUnknown, "armarx::aron::data::Descriptor::eUnknown"}
    };

    inline std::string DESCRIPTOR_TO_STRING(const data::Descriptor d)
    {
        return data::_descriptor2string.at(d);
    }

    const std::map<std::string, data::Descriptor> _string2descriptor =
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { "armarx::aron::data::Descriptor::e" + std::string(#upperType), Descriptor::e##upperType }, \
    { std::string(#upperType), Descriptor::e##upperType }, \
    { std::string(#lowerType), Descriptor::e##upperType }, \
    { std::string(#capsType), Descriptor::e##upperType },

        HANDLE_ALL_ARON_DATA
#undef RUN_ARON_MACRO
        {"armarx::aron::data::Descriptor::eUnknown", Descriptor::eUnknown},
        {"Unknown", Descriptor::eUnknown},
        {"unknown", Descriptor::eUnknown},
        {"UNKNOWN", Descriptor::eUnknown}
    };

    inline data::Descriptor STRING_TO_DESCRIPTOR(const std::string& s)
    {
        return data::_string2descriptor.at(s);
    }

    inline data::Descriptor DESCRIPTOR_FROM_STRING(const std::string& s)
    {
        return STRING_TO_DESCRIPTOR(s);
    }
}
