/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <filesystem>
#include <memory>
#include <optional>


// Base Class
#include <RobotAPI/libraries/aron/core/codegenerator/typeReader/Reader.h>

// ArmarX
#include <SimoxUtility/xml.h>
#include <ArmarXCore/core/rapidxml/wrapper/RapidXmlReader.h>

#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>
#include <RobotAPI/libraries/aron/core/codegenerator/typeReader/xml/ReaderFactory.h>

namespace armarx::aron::xmltypereader
{
    class Reader;
    typedef std::shared_ptr<Reader> ReaderPtr;

    class Reader :
        virtual public typereader::Reader<std::string>
    {
    public:
        Reader() = default;

        virtual void parseFile(const std::string&) override;
        virtual void parseFile(const std::filesystem::path&) override;

    private:
        void parse(const RapidXmlReaderPtr& node);

        std::string readCodeInclude(const RapidXmlReaderNode& node);
        std::string readAronInclude(const RapidXmlReaderNode& node);

        typenavigator::ObjectNavigatorPtr readGenerateObject(const RapidXmlReaderNode& node) const;
        typenavigator::IntEnumNavigatorPtr readGenerateIntEnum(const RapidXmlReaderNode& node) const;

    private:
        ReaderFactory factory;
    };


    namespace detail
    {
        std::optional<std::filesystem::path> resolveRelativePackagePath(const std::filesystem::path& path);
    }
}
