/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <vector>
#include <optional>

// ArmarX
#include <RobotAPI/libraries/aron/core/Config.h>
#include <RobotAPI/libraries/aron/core/Descriptor.h>

namespace armarx::aron::dataIO
{
    class ReaderInterface;
    typedef std::shared_ptr<ReaderInterface> ReaderInterfacePtr;

    class ReaderInterface
    {
    public:
        struct ReadStartDictReturnType
        {
            int elements;
            bool success;
        };
        virtual ReadStartDictReturnType readStartDict() = 0;
        virtual bool readEndDict() = 0;

        struct ReadStartListReturnType
        {
            int elements;
            bool success;
        };
        virtual ReadStartListReturnType readStartList() = 0;
        virtual bool readEndList() = 0;

        struct ReadStartNDArrayReturnType
        {
            std::vector<int> dims;
            std::string type;
            bool success;
        };
        virtual ReadStartNDArrayReturnType readStartNDArray() = 0;
        virtual void readEndNDArray(unsigned char*) = 0;

        struct ReadPrimitiveReturnType
        {
            bool success;
        };
        virtual ReadPrimitiveReturnType readStartPrimitive() = 0;
        virtual void readEndPrimitive(int&) = 0;
        virtual void readEndPrimitive(long&) = 0;
        virtual void readEndPrimitive(float&) = 0;
        virtual void readEndPrimitive(double&) = 0;
        virtual void readEndPrimitive(std::string&) = 0;
        virtual void readEndPrimitive(bool&) = 0;

        virtual std::string readKey() = 0;
        virtual void loadMember(const std::string&) = 0;

        // This method is for converters or visitors. The code generation knows what to call (e.g. readStartNDArray) since it knows the type.
        // But if the type is unknown (and null), we need a method to increase the cnt of a token to the next element
        virtual void readNull() = 0;

        // Helper functions
        virtual data::Descriptor getTypeOfNext() const = 0;
    };
}
